@@ -1,3 +1,5 @@
+from ui.Widget import Widget
+from ui.Pixmap import Pixmap
 from utils.Observable import Observable
 
 import gtk
@@ -8,24 +10,20 @@ import time
 import gc
 
 
+
 # predefined zoom levels
 _ZOOM_LEVELS = [18, 25, 33, 50, 75, 100, 150, 200,
                 300, 400, 600, 800, 1200, 1600]
 
-# the maemo platform uses 16 bits per pixel
-_BPP = gtk.gdk.get_default_root_window().get_depth()
-
 # read this many bytes at once
-_CHUNK_SIZE = 50000
-
-_BACKGROUND_COLOR = gtk.gdk.color_parse("#101010")
+_CHUNK_SIZE = 10000 #50000
 
 # the font for comments, etc.
 _FONT = "Nokia Sans Cn 22"
 
 
 
-class Image(gtk.EventBox, Observable):
+class Image(Widget, Observable):
     """
     Class for rendering images.
     """    
@@ -39,10 +37,16 @@ class Image(gtk.EventBox, Observable):
     OBS_SMOOTHING = 6  
     
 
-    def __init__(self):
+    def __init__(self, esens):
 
         # the buffer contains the pixbuf for rendering on screen
         self.__buffer = None
+        
+        # color of the background
+        self.__bg_color = gtk.gdk.color_parse("#000000")
+        
+        # offscreen drawing pixmap
+        self.__offscreen = Pixmap(None, 800, 480)
 
         # original size of the image
         self.__original_size = (0, 0)
@@ -68,6 +72,11 @@ class Image(gtk.EventBox, Observable):
         # a buffer is a tuple (pixmap, x, y, w, h)
         self.__save_unders = []
 
+        # flag for marking new images that have not been rendered to screen yet
+        self.__is_new_image = False
+               
+        self.__hi_quality_timer = None
+        
         # the currently available zoom levels (= _ZOOM_LEVELS + fitting)
         self.__zoom_levels = []
         self.__zoom_level = 4
@@ -84,74 +93,48 @@ class Image(gtk.EventBox, Observable):
         self.__loading_cancelled = False
         self.__pixbuf = None
 
+        # slide from right or left
+        self.__slide_from_right = True
         
-        gtk.EventBox.__init__(self)
-        #self.set_flags(gtk.CAN_FOCUS)
-        self.connect("expose-event", self.__on_expose)
-
-        self.__image = gtk.Image()
-        self.__image.modify_font(pango.FontDescription(_FONT))
-        self.__image.show()
-        self.add(self.__image)
-
-        # create a server-side pixmap for rendering
-        #pmap = gtk.gdk.Pixmap(None, 800, 480, _BPP)
-        #pmap.draw_rectangle(pmap.new_gc(), True, 0, 0, 800, 480)
-        #self.__image.set_from_pixmap(pmap, None)
-        #del pmap
-        self.__make_pixmap(600, 400)
+        Widget.__init__(self, esens)
 
         # create a client-side pixmap for rendering
         self.__buffer = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB,
                                        True, 8, 800, 480)
 
 
-    def __make_pixmap(self, w, h):
+    def render_this(self):
 
-        # create a server-side pixmap for rendering
-        pmap = gtk.gdk.Pixmap(None, w, h, _BPP)
-        pmap.draw_rectangle(pmap.new_gc(), True, 0, 0, w, h)
-        self.__image.set_from_pixmap(pmap, None)
-        del pmap
+        x, y = self.get_screen_pos()
+        w, h = self.get_size()
+        screen = self.get_screen()    
+        screen.copy_pixmap(self.__offscreen, x, y, x, y, w, h)
 
 
-    def __on_expose(self, src, ev):
+    def set_size(self, w, h):
+    
+        Widget.set_size(self, w, h)
 
-        self.resize()
-
-
-    def resize(self, attempt = 0):
-        """
-        Checks if the widget has been resized.
-        """
-
-        w, h = self.window.get_size()
         if ((w, h) != self.__visible_size):
-            self.__make_pixmap(w, h)        
             self.__visible_size = (w, h)
-            #if (self.__original_size != (0, 0)): self.__scale_to_fit()
-            if (self.__original_size != (0, 0)): self.zoom(self.__zoom_level)
+            if (self.__original_size != (0, 0)):
+                self.__invalidated = True
+                self._render()
+                self.__scale_to_fit()
+                #gobject.idle_add(self.__scale_to_fit)
 
-        elif (attempt < 10):
-            gobject.timeout_add(100, self.resize, attempt + 1)
+
+    def set_background(self, col):
+    
+        self.__bg_color = col
+        self.__offscreen.fill_area(0, 0, 800, 480, self.__bg_color)
         
 
-
-
     def __hi_quality_render(self):
 
-        tstamp = int(time.time() * 1000)
-        self.__timer_tstamp = tstamp
-        gobject.timeout_add(1000, self.__hi_quality_timer, tstamp)
-
-
-    def __hi_quality_timer(self, tstamp):
-
-        if (tstamp != self.__timer_tstamp): return
-
-        self.update_observer(self.OBS_SMOOTHING)
-        self.__invalidated = True
-        gobject.idle_add(self._render, True)
+       self.update_observer(self.OBS_SMOOTHING)
+       self.__invalidated = True
+       self._render(True)
 
 
     def scroll_to(self, x, y):
@@ -162,7 +145,8 @@ class Image(gtk.EventBox, Observable):
         y = max(height / 2, min(y, vheight - height / 2))
         self.__center_pos = (x, y)
 
-        gobject.idle_add(self._render, False)
+        self._render(False)
+        #gobject.idle_add(self._render, False)
 
 
     def scroll_by(self, dx, dy):
@@ -225,7 +209,7 @@ class Image(gtk.EventBox, Observable):
         self.__virtual_size = (int(bufwidth * self.__zoom_value),
                                int(bufheight * self.__zoom_value))
 
-        for i in range(3): gc.collect()
+        #for i in range(3): gc.collect()
         self.__invalidated = True
 
         self.update_observer(self.OBS_ZOOMING,
@@ -235,122 +219,122 @@ class Image(gtk.EventBox, Observable):
         #                                       cy * self.__zoom_value)
 
 
-    def __push_save_under(self, x, y, w, h):
-        """
-        Saves the given area so that it can be restored later.
-        """
-
-        # it's all happening on the server-side, maybe even hw-accelerated
-        pmap, nil = self.__image.get_pixmap()
-        saved = gtk.gdk.Pixmap(None, w, h, _BPP)
-        saved.draw_drawable(pmap.new_gc(), pmap,
-                            x, y, 0, 0, w, h)
-        self.__save_unders.append((saved, x, y, w, h))
+   # def __push_save_under(self, x, y, w, h):
+   #     """
+   #     Saves the given area so that it can be restored later.
+   #     """
+   #
+   #     # it's all happening on the server-side, maybe even hw-accelerated
+   #     pmap, nil = self.__image.get_pixmap()
+   #     saved = gtk.gdk.Pixmap(None, w, h, _BPP)
+   #     saved.draw_drawable(pmap.new_gc(), pmap,
+   #                         x, y, 0, 0, w, h)
+   #     self.__save_unders.append((saved, x, y, w, h))
 
 
-    def __pop_save_under(self):
-        """
-        Restores a saved area.
-        """
-
-        # it's all happening on the server-side, maybe even hw-accelerated
-        pmap, nil = self.__image.get_pixmap()
-        if (self.__save_unders):
-            saved, x, y, w, h = self.__save_unders.pop()
-            pmap.draw_drawable(pmap.new_gc(), saved,
-                               0, 0, x, y, w, h)
+    #def __pop_save_under(self):
+    #    """
+    #    Restores a saved area.
+    #    """
+    #
+    #    # it's all happening on the server-side, maybe even hw-accelerated
+    #    pmap, nil = self.__image.get_pixmap()
+    #    if (self.__save_unders):
+    #        saved, x, y, w, h = self.__save_unders.pop()
+    #        pmap.draw_drawable(pmap.new_gc(), saved,
+    #                           0, 0, x, y, w, h)
 
 
 
-    def _render_text(self, text, posx, posy, draw_box):
-        """
-        Renders the given text on screen. The position is
-        a percentage value of the widget size.
-        """
-
-        w, h = self.__visible_size
-
-        context = self.__image.get_pango_context()
-        layout = pango.Layout(context)
-        layout.set_width(w * pango.SCALE)
-        layout.set_text(text)
-
-        layout_w, layout_h = layout.get_pixel_size()
-        x = layout_w / 2 + int((w - layout_w) * posx - layout_w / 2)
-        y = layout_h / 2 + int((h - layout_h) * posy - layout_h / 2)
-
-        self.__push_save_under(x - 3, y - 3, layout_w + 6, layout_h + 6)
-
-        pmap, nil = self.__image.get_pixmap()
-        gc = pmap.new_gc()
-        gc.set_foreground(self.__image.get_colormap().alloc_color("black"))
-
-        if (draw_box):
-            pbuf = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB,
-                                  True, 8, layout_w + 4, layout_h + 4)
-            pbuf.fill(0xffffffa0)
-            pmap.draw_pixbuf(None, pbuf, 0, 0, x - 2, y - 2, layout_w + 4, layout_h + 4,
-                             gtk.gdk.RGB_DITHER_NONE, 0, 0)
-            del pbuf
-            pmap.draw_rectangle(gc, False, x - 3, y - 3, layout_w + 5, layout_h + 5)
-        
-        pmap.draw_layout(gc, x, y, layout, gtk.gdk.color_parse("black"))
-
-        # only redraw the affected portion of the screen
-        self.__image.queue_draw_area(x - 3, y - 3, layout_w + 6, layout_h + 6)
+    #def _render_text(self, text, posx, posy, draw_box):
+    #    """
+    #    Renders the given text on screen. The position is
+    #    a percentage value of the widget size.
+    #    """
+    #
+    #    w, h = self.__visible_size
+    #
+    #    context = self.__image.get_pango_context()
+    #    layout = pango.Layout(context)
+    #    layout.set_width(w * pango.SCALE)
+    #    layout.set_text(text)
+    #
+    #    layout_w, layout_h = layout.get_pixel_size()
+    #    x = layout_w / 2 + int((w - layout_w) * posx - layout_w / 2)
+    #    y = layout_h / 2 + int((h - layout_h) * posy - layout_h / 2)
+    #
+    #    self.__push_save_under(x - 3, y - 3, layout_w + 6, layout_h + 6)
+    #
+    #    pmap, nil = self.__image.get_pixmap()
+    #    gc = pmap.new_gc()
+    #    gc.set_foreground(self.__image.get_colormap().alloc_color("black"))
+    #
+    #    if (draw_box):
+    #        pbuf = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB,
+    #                              True, 8, layout_w + 4, layout_h + 4)
+    #        pbuf.fill(0xffffffa0)
+    #        pmap.draw_pixbuf(None, pbuf, 0, 0, x - 2, y - 2, layout_w + 4, layout_h + 4,
+    #                         gtk.gdk.RGB_DITHER_NONE, 0, 0)
+    #        del pbuf
+    #        pmap.draw_rectangle(gc, False, x - 3, y - 3, layout_w + 5, layout_h + 5)
+    #    
+    #    pmap.draw_layout(gc, x, y, layout, gtk.gdk.color_parse("black"))
+    #
+    #    # only redraw the affected portion of the screen
+    #    self.__image.queue_draw_area(x - 3, y - 3, layout_w + 6, layout_h + 6)
 
 
-    def _render_bar(self, posy1, posy2):
-        """
-        Renders a colored bar on screen at the given position.
-        The position is a percentage value of the widget size.
-        """
-
-        w, h = self.__visible_size
-        y1 = int(h * posy1)
-        y2 = int(h * posy2)
-        height = abs(y1 - y2)
-
-        pmap, nil = self.__image.get_pixmap()
-        pbuf = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB,
-                              True, 8, w, height)
-        pbuf.fill(0xffffffa0)
-        pmap.draw_pixbuf(None, pbuf, 0, 0, 0, y1, w, height,
-                         gtk.gdk.RGB_DITHER_NONE, 0, 0)
-        del pbuf
-
-        # only redraw the affected portion of the screen
-        self.__image.queue_draw_area(0, y1, w, height)
+    #def _render_bar(self, posy1, posy2):
+    #    """
+    #    Renders a colored bar on screen at the given position.
+    #    The position is a percentage value of the widget size.
+    #    """
+    #
+    #    w, h = self.__visible_size
+    #    y1 = int(h * posy1)
+    #    y2 = int(h * posy2)
+    #    height = abs(y1 - y2)
+    #
+    #    pmap, nil = self.__image.get_pixmap()
+    #    pbuf = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB,
+    #                          True, 8, w, height)
+    #    pbuf.fill(0xffffffa0)
+    #    pmap.draw_pixbuf(None, pbuf, 0, 0, 0, y1, w, height,
+    #                     gtk.gdk.RGB_DITHER_NONE, 0, 0)
+    #    del pbuf
+    #
+    #    # only redraw the affected portion of the screen
+    #    self.__image.queue_draw_area(0, y1, w, height)
 
         
 
-    def _render_icon(self, icon, posx, posy):
-        """
-        Renders the given icon on screen. The position is
-        a percentage value of the widget size.
-        """
-
-        w, h = self.__visible_size
-        icon_w = icon.get_width()
-        icon_h = icon.get_height()
-        x = icon_w / 2 + int((w - icon_w) * posx - icon_w / 2)
-        y = icon_h / 2 + int((h - icon_h) * posy - icon_h / 2)
-
-        self.__push_save_under(x, y, icon_w, icon_h)
-
-        pmap, nil = self.__image.get_pixmap()
-        pmap.draw_pixbuf(None, icon,
-                         0, 0, x, y, icon_w, icon_h,
-                         gtk.gdk.RGB_DITHER_NONE, 0, 0)
-
-        # only redraw the affected portion of the screen
-        self.__image.queue_draw_area(x, y, icon_w, icon_h)
+    #def _render_icon(self, icon, posx, posy):
+    #    """
+    #    Renders the given icon on screen. The position is
+    #    a percentage value of the widget size.
+    #    """
+    #
+    #    w, h = self.__visible_size
+    #    icon_w = icon.get_width()
+    #    icon_h = icon.get_height()
+    #    x = icon_w / 2 + int((w - icon_w) * posx - icon_w / 2)
+    #    y = icon_h / 2 + int((h - icon_h) * posy - icon_h / 2)
+    #
+    #    self.__push_save_under(x, y, icon_w, icon_h)
+    #
+    #    pmap, nil = self.__image.get_pixmap()
+    #    pmap.draw_pixbuf(None, icon,
+    #                     0, 0, x, y, icon_w, icon_h,
+    #                     gtk.gdk.RGB_DITHER_NONE, 0, 0)
+    #
+    #    # only redraw the affected portion of the screen
+    #    self.__image.queue_draw_area(x, y, icon_w, icon_h)
 
 
-    def _render_clear(self):
-
-        while (self.__save_unders):
-            self.__pop_save_under()
+    #def _render_clear(self):
+    #
+    #    while (self.__save_unders):
+    #        self.__pop_save_under()
         
 
     def _render(self, high_quality = False):
@@ -358,6 +342,9 @@ class Image(gtk.EventBox, Observable):
         Renders the visible area of the image.
         """
 
+        x, y = self.get_screen_pos()
+        screen = self.get_screen()
+
         offx, offy = self.__get_offset()
         width, height = self.__visible_size
         vwidth, vheight = self.__virtual_size
@@ -390,34 +377,61 @@ class Image(gtk.EventBox, Observable):
             src_h = height + dy
             areas.append((0, 0, width, -dy))
 
-        self._render_clear()
+        #self._render_clear()
 
-        pmap, nil = self.__image.get_pixmap()
         if (self.__invalidated):
             # draw full area
             areas = [(0, 0, width, height)]
             self.__invalidated = False
             
-            if (vwidth < width or vheight < height):
-                gc = pmap.new_gc()
-                gc.set_foreground(_BACKGROUND_COLOR)
-                pmap.draw_rectangle(gc, True, 0, 0, width, height)
+            if (vwidth < width):
+                bw = (width - vwidth) / 2
+                self.__offscreen.fill_area(x, y, bw, height, self.__bg_color)
+                self.__offscreen.fill_area(x + width - bw - 1, y, bw + 1, height,
+                                           self.__bg_color)
+            
+            if (vheight < height):
+                bh = (height - vheight) / 2
+                self.__offscreen.fill_area(x, y, width, bh, self.__bg_color)
+                self.__offscreen.fill_area(x, y + height - bh - 1, width, bh + 1,
+                                           self.__bg_color)
                 
         else:
             # copy on the server-side (this is our simple trick for
             # fast scrolling!)
-            pmap.draw_drawable(pmap.new_gc(), pmap,
-                               src_x, src_y, dest_x, dest_y, src_w, src_h)
+            self.__offscreen.copy_pixmap(screen,
+                                         x + src_x, y + src_y,
+                                         x + dest_x, y + dest_y,
+                                         src_w, src_h)
             
         # render borders
         for rx, ry, rw, rh in areas:
             self.__render_area(rx, ry, rw, rh, high_quality)
 
+        # wait for VBL
+        #omapfb.sync_gfx()
+
+        # copy to screen
+        if (self.may_render()):
+            if (self.__is_new_image):
+                self.__is_new_image = False
+                self.fx_slide_in()
+            else:
+                screen.copy_pixmap(self.__offscreen, x, y, x, y, width, height)            
+            
+            if (not high_quality):
+                if (self.__hi_quality_timer):
+                    gobject.source_remove(self.__hi_quality_timer)
+                    
+                self.__hi_quality_timer = \
+                  gobject.timeout_add(1000, self.__hi_quality_render)
+            else:
+                self.__hi_quality_timer = None
+
         self.__previous_offset = (offx, offy)
         
         self.update_observer(self.OBS_RENDERED)
-        self.__image.queue_draw()
-        if (not high_quality): self.__hi_quality_render()
+
 
         
 
@@ -466,17 +480,15 @@ class Image(gtk.EventBox, Observable):
         # copy from client-side pixmap to server-side pixmap
         # I wish we could avoid this time-consuming step... (server-side
         # scaling would be nice!)
-        pmap, nil = self.__image.get_pixmap()
 
         # maybe we have to center the image?
         if (vwidth < width or vheight < height):
             if (vwidth < width): rx += (width - vwidth) / 2
             if (vheight < height): ry += (height - vheight) / 2
-        
-        pmap.draw_pixbuf(None, destpbuf,
-                         0, 0, rx, ry, destwidth, destheight,
-                         gtk.gdk.RGB_DITHER_NONE, 0, 0)
-        
+
+        x, y = self.get_screen_pos()
+        self.__offscreen.draw_pixbuf(destpbuf, x + rx, y + ry)        
+       
 
 
 
@@ -488,12 +500,51 @@ class Image(gtk.EventBox, Observable):
 
         if (filename != self.__current_filename):
             self.__loading_cancelled = True
+            
+            if (self.__hi_quality_timer):
+                gobject.source_remove(self.__hi_quality_timer)            
+
             self.update_observer(self.OBS_BEGIN_LOADING, filename)
             
-            # timeout for the cancelation to take effect
-            gobject.timeout_add(50, self.__load_img, filename)
+            self.__load_img_at_once(filename)
+            self.__finish_loading()
 
 
+    def __load_img_at_once(self, filename):
+        """
+        Loads the image.
+        """
+
+        try:
+            self.__loader.close()
+        except:
+            pass
+        self.__loader = gtk.gdk.PixbufLoader()
+        self.__loader.connect("size-prepared", self.__on_check_size)
+        self.__current_filename = filename
+        
+        fd = open(filename, "r")
+        # determine file size
+        fd.seek(0, 2)
+        size = fd.tell()
+        fd.seek(0)        
+
+        size_read = 0
+        while (size > 0):
+            data = fd.read(_CHUNK_SIZE)
+            size_read += len(data)
+
+            if (data):
+                if (self.__loader): self.__loader.write(data)
+            else:
+                break
+
+            self.update_observer(self.OBS_PROGRESS, size_read, size)
+        #end while
+        
+
+        self.__loader.close()
+
 
     def __load_img(self, filename):
         """
@@ -504,7 +555,7 @@ class Image(gtk.EventBox, Observable):
             self.__banner.close()
 
         if (not os.path.exists(filename)):
-            filename = "" #pixmaps.get_path("image-missing.png")
+            filename = ""
 
         try:
             self.__loader.close()
@@ -518,11 +569,6 @@ class Image(gtk.EventBox, Observable):
         size = fd.tell()
         fd.seek(0)
 
-        # collect three generations of garbage
-        for i in range(3): gc.collect()
-
-        #self.__banner = ProgressBanner(self,
-        #                "%s" % os.path.basename(filename), size)
         self.__loading_cancelled = False
         self.__current_filename = filename
         
@@ -530,9 +576,6 @@ class Image(gtk.EventBox, Observable):
         while (True and size > 0):
             data = fd.read(_CHUNK_SIZE)
             size_read += len(data)
-            
-            # this runs the gtk mainiteration and thus allows user interaction
-            #if (self.__banner): self.__banner.progress(_CHUNK_SIZE)            
 
             self.update_observer(self.OBS_PROGRESS, size_read, size)
             while (gtk.events_pending()): gtk.main_iteration()
@@ -551,8 +594,6 @@ class Image(gtk.EventBox, Observable):
             try:
                 self.__loader.close()
             except:
-                pass #hildon.hildon_banner_show_information(self, gtk.STOCK_DIALOG_ERROR,
-                     #                                 "Not a valid image")
                 self.__loading_cancelled = True
 
             if (not self.__loading_cancelled): self.__finish_loading()
@@ -573,6 +614,18 @@ class Image(gtk.EventBox, Observable):
         #print "REF", self.__loader.__grefcount__
 
         pbuf = self.__loader.get_pixbuf()
+        
+        # properly support images with alpha channel
+        if (pbuf.get_has_alpha()):
+            pbuf2 = pbuf.composite_color_simple(pbuf.get_width(),
+                                                pbuf.get_height(),
+                                                gtk.gdk.INTERP_NEAREST, 0xff,
+                                                32, 0, 0)
+            del pbuf
+            pbuf = pbuf2
+            del pbuf2
+        #end if
+            
 
         # automatically rotate images in portrait format
         if (pbuf.get_width() < pbuf.get_height()):
@@ -590,12 +643,13 @@ class Image(gtk.EventBox, Observable):
         w, h = self.__pixbuf.get_width(), self.__pixbuf.get_height()
         self.__original_size = (w, h)
 
+        self.update_observer(self.OBS_END_LOADING)
+        
+        self.__is_new_image = True
         self.__scale_to_fit()
 
         # collect three generations of garbage
-        for i in range(3): gc.collect()
-
-        self.update_observer(self.OBS_END_LOADING)
+        #for i in range(3): gc.collect()
 
 
     def __scale_to_fit(self):
@@ -648,3 +702,45 @@ class Image(gtk.EventBox, Observable):
 
         if (factor != 1):
             loader.set_size(int(width * factor), int(height * factor))
+      
+        
+    def slide_from_left(self):
+    
+        self.__slide_from_right = False
+        
+        
+    def slide_from_right(self):
+    
+        self.__slide_from_right = True
+        
+        
+    def fx_slide_in(self, wait = True):
+    
+        import threading
+
+        STEP = 40
+        x, y = self.get_screen_pos()
+        w, h = self.get_size()
+        screen = self.get_screen()
+
+        finished = threading.Event()
+        
+        def f(i):
+            i = min(i, w)
+            if (self.__slide_from_right):
+                screen.copy_buffer(screen, x + STEP, y, x, y, w - STEP, h)
+                screen.copy_pixmap(self.__offscreen, x + i, y, x + w - STEP, y,
+                                   STEP, h)
+            else:
+                screen.copy_buffer(screen, x, y, x + STEP, y, w - STEP, h)
+                screen.copy_pixmap(self.__offscreen, x + w - STEP - i, y, x, y,
+                                   STEP, h)
+            
+            if (i < w - STEP):
+                gobject.timeout_add(5, f, i + STEP)
+            else:
+                finished.set()
+
+        f(0)
+        while (wait and not finished.isSet()): gtk.main_iteration()
+        
